package com.neurotec.ncheck.sdk.samples.dummyscanner;

import android.graphics.Bitmap;
import android.graphics.Point;
import android.util.Range;
import android.widget.Toast;

import com.neurotec.ncheck.device.CaptureDataType;
import com.neurotec.ncheck.device.CaptureDeviceStatus;
import com.neurotec.ncheck.device.DisplayMessageType;
import com.neurotec.ncheck.sdk.CaptureDeviceService;

import java.nio.ByteBuffer;
import java.util.Locale;
import java.util.Random;
import java.util.Timer;
import java.util.TimerTask;

/**
 * Created by rajeevan on 9/13/2017.
 */

public class DummyFingerprintService extends CaptureDeviceService {

    static String SERVICE_NAME = "Dummy fingerprint service";

    public static CaptureDataType[] DeviceTypes = {CaptureDataType.FINGERPRINT, CaptureDataType.SENSOR};
    Point mCurrentImageSize = new Point(512, 512);
    Point[] mImageSizes = new Point[]{mCurrentImageSize, new Point(256, 256)};
    Timer timerCapture = null;
    Timer timerSense = null;
    boolean isCapturing = false;
    boolean isSensing = false;

    @Override
    public void showResults(String name, int inout, String time1, String time2) {
        if (inout < 0)
            Toast.makeText(getApplicationContext(), name + " \r\nOUT: " + time2 + " \r\nIN: " + time1, Toast.LENGTH_SHORT).show();
        else if (inout > 0)
            Toast.makeText(getApplicationContext(), name + " \r\nIN: " + time1, Toast.LENGTH_SHORT).show();
        else
            Toast.makeText(getApplicationContext(), name + " \r\n: " + time1, Toast.LENGTH_SHORT).show();

    }

    @Override
    synchronized public boolean capture(CaptureDataType captureDataType, Point point) {
        if (!isCapturing) {
            isCapturing = true;
            timerCapture = new Timer();
            if (point != null)
                mCurrentImageSize = point;
            else
                mCurrentImageSize = mImageSizes[0];
            new CaptureTask().run();
            return true;
        }
        return false;
    }

    @Override
    synchronized public void cancelCapture(CaptureDataType captureType) {
        if (isCapturing) {
            timerCapture.cancel();
            isCapturing = false;
        }
    }

    @Override
    public void start() {
        cancelCapture(CaptureDataType.FINGERPRINT);
        statusChanged(CaptureDeviceStatus.READY);
    }

    @Override
    public void stop() {
        cancelCapture(CaptureDataType.FINGERPRINT);
        statusChanged(CaptureDeviceStatus.NOT_READY);
    }

    @Override
    public String getName() {
        return SERVICE_NAME;
    }

    @Override
    public CaptureDataType[] getCaptureTypes() {
        return DeviceTypes;
    }

    @Override
    public Point[] getImageSizes() {
        return mImageSizes;
    }

    @Override
    public void showMessage(DisplayMessageType type, String msg) {
        // Message type is not considered.
        Toast.makeText(getApplicationContext(), msg, Toast.LENGTH_SHORT).show();
    }

    @Override
    synchronized public boolean sense(int index) {
        if (!isSensing) {
            isSensing = true;
            timerSense = new Timer();
            new SenseTask().run();
            return true;
        }
        return false;
    }

    @Override
    synchronized public void cancelSense(int index) {
        if (isSensing) {
            timerSense.cancel();
            isSensing = false;
        }
        isSensing = false;
    }

    class CaptureTask extends TimerTask {
        @Override
        public void run() {
            int pixelCount = mCurrentImageSize.x * mCurrentImageSize.y;
            byte[] b = new byte[pixelCount];
            new Random().nextBytes(b);
            byte[] bits = new byte[pixelCount * 4]; //That's where the RGBA array goes.
            for (int i = 0; i < pixelCount; i++) {
                bits[i * 4] = bits[i * 4 + 1] = bits[i * 4 + 2] = (byte) b[i]; //Invert the source bits
                bits[i * 4 + 3] = (byte) 0xff; // the alpha.
            }

            Bitmap bm = Bitmap.createBitmap(mCurrentImageSize.x, mCurrentImageSize.y, Bitmap.Config.ARGB_8888);
            bm.copyPixelsFromBuffer(ByteBuffer.wrap(bits));
            //Send dumy fingerprint
            sendCapturedData(CaptureDataType.FINGERPRINT, bm, null, null);
            int delay = (5 + new Random().nextInt(5)) * 200;
            try {
                timerCapture.schedule(new CaptureTask(), delay);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    private final Random random = new Random();


    private static final double TEMPRATURE_THRESHOLD = 37.5;
    private static final double TEMPRATURE_MIN = 34.5;
    private static final double TEMPRATURE_MAX = 41.5;
    private static final double PRECISION = 0.1;
    private static final String TEMPRATURE_MESSAGE = "%.2f ℃";
    private static final String NO_OBJECT = "No object!";
    private static final String TOO_HOT = "Too hot!";
    private static final Range<Double> TEMP_RANGE = new Range<>(TEMPRATURE_MIN, TEMPRATURE_MAX);

    class SenseTask extends TimerTask {
        @Override
        public void run() {

            double temprature = random.nextDouble()* 9 + 33.5;
            //double sensorValue = Math.min(threshold + 10, Math.max(new Random().nextGaussian() * 5 + previousValue, threshold - 50));
            //double sensorValue = Math.min(threshold + 20, Math.max(increasse ? previousValue + 1 : previousValue - 1, threshold - 20));
            if (temprature < TEMPRATURE_MIN) {
                sendSensorDataUpdate(0, TEMP_RANGE, PRECISION, TEMPRATURE_THRESHOLD, NO_OBJECT, temprature);
            } else if (temprature > TEMPRATURE_MAX) {
                sendSensorDataUpdate(0, TEMP_RANGE, PRECISION, TEMPRATURE_THRESHOLD, TOO_HOT, temprature);
            } else {
                sendSensorDataUpdate(0, TEMP_RANGE, PRECISION, TEMPRATURE_THRESHOLD, String.format(Locale.US, TEMPRATURE_MESSAGE, temprature), temprature);
            }

            int delay = (5 + new Random().nextInt(5)) * 200;

            try {
                timerSense.schedule(new SenseTask(), delay);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }
}
