﻿using Microsoft.Win32;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Prism.Mvvm;
using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Text;
using System.Windows;
using System.Windows.Input;

namespace APICLientSample.ViewModels
{
    public class MainWindowViewModel : BindableBase
    {
        private string _token = null;
        private string _title = "NCheck API Sample";
        public string Title
        {
            get { return _title; }
            set { SetProperty(ref _title, value); }
        }
        private string _serverUrl = APICLientSample.Properties.Settings.Default.server;
        public string ServerUrl
        {
            get { return string.IsNullOrWhiteSpace(_serverUrl) ? _serverUrl : _serverUrl.TrimEnd('/'); }
            set {
                SetProperty(ref _serverUrl, value);
                APICLientSample.Properties.Settings.Default.server = value;
                APICLientSample.Properties.Settings.Default.Save();
            }
        }
        private string _key = APICLientSample.Properties.Settings.Default.key;
        public string Key
        {
            get { return _key; }
            set
            {
                SetProperty(ref _key, value);
                APICLientSample.Properties.Settings.Default.key = value;
                APICLientSample.Properties.Settings.Default.Save();
            }
        }
        private string _secret = APICLientSample.Properties.Settings.Default.secret;
        public string Secret
        {
            get { return _secret; }
            set
            {
                SetProperty(ref _secret, value);
                APICLientSample.Properties.Settings.Default.secret = value;
                APICLientSample.Properties.Settings.Default.Save();
            }
        }

        private DateTime _syncDate = DateTime.Now;
        public DateTime SyncDate
        {
            get { return _syncDate; }
            set { SetProperty(ref _syncDate, value); }
        }
        private Employee _employee = new Employee();
        public Employee Employee
        {
            get { return _employee; }
            set { SetProperty(ref _employee, value); }
        }

        private AttendanceEvent _attendanceEvent = new AttendanceEvent();
        public AttendanceEvent AttendanceEvent
        {
            get { return _attendanceEvent; }
            set { SetProperty(ref _attendanceEvent, value); }
        }

        private AttendanceEvent _selectedEvent;
        public AttendanceEvent SelectedEvent
        {
            get { return _selectedEvent; }
            set { SetProperty(ref _selectedEvent, value); }
        }


        private List<AttendanceEvent> _attendanceEventList;
        public List<AttendanceEvent> AttendanceEventList
        {
            get { return _attendanceEventList; }
            set { SetProperty(ref _attendanceEventList, value); }
        }

        private void SetSSLValidator(WebRequest req)
        {
            ((HttpWebRequest)req).ServerCertificateValidationCallback +=
            (sender, cert, chain, error) =>
            {
                return true;
            };

        }

        private ICommand _connectCommand;
        public ICommand ConnectCommand
        {
            get
            {
                return _connectCommand ?? (_connectCommand = new CommandHandler(() =>
                {
                    try
                    {
                        if (string.IsNullOrWhiteSpace(_serverUrl) || string.IsNullOrWhiteSpace(_key) || string.IsNullOrWhiteSpace(_secret))
                        {
                            MessageBox.Show("Invalid data.");
                            return;
                        };

                        WebRequest req = WebRequest.Create(_serverUrl + "/oauth/token");
                        SetSSLValidator(req);
                        req.Method = "POST";
                        req.Headers["Authorization"] = "Basic " + Convert.ToBase64String(Encoding.Default.GetBytes("ncheck:"));
                        req.ContentType = "application/x-www-form-urlencoded";
                        var postData = "grant_type=password&username=" + _key + "&password=" + _secret;
                        var data = Encoding.ASCII.GetBytes(postData);
                        req.ContentLength = data.Length;
                        using (var stream = req.GetRequestStream())
                        {
                            stream.Write(data, 0, data.Length);
                        }
                        HttpWebResponse resp = req.GetResponse() as HttpWebResponse;
                        if (resp.StatusCode == HttpStatusCode.OK)
                        {
                            var responseString = new StreamReader(resp.GetResponseStream()).ReadToEnd();
                            JObject json = JObject.Parse(responseString);
                            _token = json["access_token"].Value<string>();
                            MessageBox.Show(string.Format("Access token aquired.:{0}).", _token));
                        }
                        else
                        {
                            MessageBox.Show(string.Format("Error:{0}({1}).", resp.StatusDescription, resp.StatusCode), "Error");
                        }
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(string.Format(ex.StackTrace), "Error");
                    }
                }));
            }
        }

        private ICommand _newUserCommand;
        public ICommand NewUserCommand
        {
            get
            {
                return _newUserCommand ?? (_newUserCommand = new CommandHandler(() =>
                {
                    Employee = new Employee();
                }));
            }
        }

        private ICommand _loadUserCommand;
        public ICommand LoadUserCommand
        {
            get
            {
                return _loadUserCommand ?? (_loadUserCommand = new CommandHandler(() =>
                {
                    try
                    {
                        if (string.IsNullOrWhiteSpace(_token))
                        {
                            MessageBox.Show("Not connected");
                            return;
                        }
                        if (string.IsNullOrWhiteSpace(_employee.employeeCode))
                        {
                            MessageBox.Show("Employee code is required");
                            return;
                        }
                        WebRequest req = WebRequest.Create(_serverUrl + "/api/ncheck/user?code=" + _employee.employeeCode);
                        SetSSLValidator(req);
                        req.Method = "GET";
                        req.Headers["Authorization"] = "Bearer " + _token;
                        req.ContentType = "application/json";
                        HttpWebResponse resp = req.GetResponse() as HttpWebResponse;
                        if (resp.StatusCode == HttpStatusCode.OK)
                        {
                            var result = new StreamReader(resp.GetResponseStream()).ReadToEnd();
                            var apiResult = JsonConvert.DeserializeObject<APIResult<Employee>>(result);
                            if (apiResult.statusCode == "SUCCESS") Employee = apiResult.returnValue;
                            MessageBox.Show(string.Format("Get user completed with code : {0} \r\n description : {1} ", apiResult.statusCode, apiResult.statusDescription));
                        }
                        else
                        {
                            MessageBox.Show(string.Format("Error:{0}({1}).", resp.StatusDescription, resp.StatusCode), "Error");
                        }
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(string.Format(ex.StackTrace), "Error");
                    }
                }));
            }
        }

        private ICommand _updateUserCommand;
        public ICommand UpdateUserCommand
        {
            get
            {
                return _updateUserCommand ?? (_updateUserCommand = new CommandHandler(() =>
                {
                    try
                    {
                        if (string.IsNullOrWhiteSpace(_token))
                        {
                            MessageBox.Show("Not connected");
                            return;
                        }
                        if (string.IsNullOrWhiteSpace(_employee.employeeCode))
                        {
                            MessageBox.Show("Employee code is required");
                            return;
                        }
                        WebRequest req = WebRequest.Create(_serverUrl + "/api/ncheck/user");
                        SetSSLValidator(req);
                        req.Method = "POST";
                        req.Headers["Authorization"] = "Bearer " + _token;
                        req.ContentType = "application/json; charset=utf-8";
                        var postData = JsonConvert.SerializeObject(Employee);
                        var data = Encoding.UTF8.GetBytes(postData);
                        req.ContentLength = data.Length;
                        using (var stream = req.GetRequestStream())
                        {
                            stream.Write(data, 0, data.Length);
                        }
                        HttpWebResponse resp = req.GetResponse() as HttpWebResponse;
                        if (resp.StatusCode == HttpStatusCode.OK)
                        {
                            var result = new StreamReader(resp.GetResponseStream()).ReadToEnd();
                            var apiResult = JsonConvert.DeserializeObject<APIResult<Employee>>(result);
                            MessageBox.Show(string.Format("Update user completed with code {0}, description {1} ", apiResult.statusCode, apiResult.statusDescription));
                        }
                        else
                        {
                            MessageBox.Show(string.Format("Error:{0}({1}).", resp.StatusDescription, resp.StatusCode), "Error");
                        }
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(string.Format(ex.StackTrace), "Error");
                    }

                }));
            }
        }

        private ICommand _updateFacesCommand;
        public ICommand UpdateFacesCommand
        {
            get
            {
                return _updateFacesCommand ?? (_updateFacesCommand = new CommandHandler(() =>
                {
                    try
                    {
                        if (string.IsNullOrWhiteSpace(_token))
                        {
                            MessageBox.Show("Not connected");
                            return;
                        }
                        if (string.IsNullOrWhiteSpace(_employee.employeeCode))
                        {
                            MessageBox.Show("Employee code is required");
                            return;
                        }
                        OpenFileDialog openFileDialog = new OpenFileDialog();
                        openFileDialog.Filter = "PNG files (*.png)|*.png|Jpeg files (*.jpg)|*.jpg|All files (*.*)|*.*";
                        openFileDialog.FilterIndex = 2;
                        openFileDialog.RestoreDirectory = true;
                        openFileDialog.Multiselect = true;
                        openFileDialog.Title = " Select face images";
                        List<Biometric> faces = new List<Biometric>();
                        if (openFileDialog.ShowDialog() == true)
                        {

                            foreach (String file in openFileDialog.FileNames)
                            {
                                faces.Add(new Biometric() { modality = "face", image = File.ReadAllBytes(file) });
                            }
                        }
                        if (faces.Count == 0)
                        {
                            MessageBox.Show("Faces were not selected.");
                            return;
                        }
                        WebRequest req = WebRequest.Create(_serverUrl + "/api/ncheck/biometric?code=" + _employee.employeeCode);
                        SetSSLValidator(req);
                        req.Method = "POST";
                        req.Headers["Authorization"] = "Bearer " + _token;
                        req.ContentType = "application/json; charset=utf-8";
                        var postData = JsonConvert.SerializeObject(faces);
                        var data = Encoding.UTF8.GetBytes(postData);
                        req.ContentLength = data.Length;
                        using (var stream = req.GetRequestStream())
                        {
                            stream.Write(data, 0, data.Length);
                        }
                        HttpWebResponse resp = req.GetResponse() as HttpWebResponse;
                        if (resp.StatusCode == HttpStatusCode.OK)
                        {
                            var result = new StreamReader(resp.GetResponseStream()).ReadToEnd();
                            var apiResult = JsonConvert.DeserializeObject<APIResult<IList<APIResult<Object>>>>(result);
                            MessageBox.Show(string.Format("Update user biometric completed with code {0}, description {1} ", apiResult.statusCode, apiResult.statusDescription));
                        }
                        else
                        {
                            MessageBox.Show(string.Format("Error:{0}({1}).", resp.StatusDescription, resp.StatusCode), "Error");
                        }
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(string.Format(ex.StackTrace), "Error");
                    }

                }));
            }
        }
        private ICommand _clearFacesCommand;
        public ICommand ClearFacesCommand
        {
            get
            {
                return _clearFacesCommand ?? (_clearFacesCommand = new CommandHandler(() =>
                {
                    try
                    {
                        if (string.IsNullOrWhiteSpace(_token))
                        {
                            MessageBox.Show("Not connected");
                            return;
                        }
                        if (string.IsNullOrWhiteSpace(_employee.employeeCode))
                        {
                            MessageBox.Show("Employee code is required");
                            return;
                        }
                        WebRequest req = WebRequest.Create(_serverUrl + "/api/ncheck/biometric/" + _employee.employeeCode + "/");
                        SetSSLValidator(req);
                        req.Method = "DELETE";
                        req.Headers["Authorization"] = "Bearer " + _token;
                        HttpWebResponse resp = req.GetResponse() as HttpWebResponse;
                        if (resp.StatusCode == HttpStatusCode.OK)
                        {
                            var result = new StreamReader(resp.GetResponseStream()).ReadToEnd();
                            var apiResult = JsonConvert.DeserializeObject<APIResult<Employee>>(result);
                            MessageBox.Show(string.Format("Delete user biometric completed with code {0}, description {1} ", apiResult.statusCode, apiResult.statusDescription));
                        }
                        else
                        {
                            MessageBox.Show(string.Format("Error:{0}({1}).", resp.StatusDescription, resp.StatusCode), "Error");
                        }
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(string.Format(ex.StackTrace), "Error");
                    }

                }));
            }
        }

        private ICommand _deleteUserCommand;
        public ICommand DeleteUserCommand
        {
            get
            {
                return _deleteUserCommand ?? (_deleteUserCommand = new CommandHandler(() =>
                {
                    try
                    {
                        if (string.IsNullOrWhiteSpace(_token))
                        {
                            MessageBox.Show("Not connected");
                            return;
                        }
                        if (string.IsNullOrWhiteSpace(_employee.employeeCode))
                        {
                            MessageBox.Show("Employee code is required");
                            return;
                        }
                        WebRequest req = WebRequest.Create(_serverUrl + "/api/ncheck/user/" + _employee.employeeCode + "/");
                        SetSSLValidator(req);
                        req.Method = "DELETE";
                        req.Headers["Authorization"] = "Bearer " + _token;
                        req.ContentType = "application/json; charset=utf-8";
                        HttpWebResponse resp = req.GetResponse() as HttpWebResponse;
                        if (resp.StatusCode == HttpStatusCode.OK)
                        {
                            var result = new StreamReader(resp.GetResponseStream()).ReadToEnd();
                            var apiResult = JsonConvert.DeserializeObject<APIResult<Employee>>(result);
                            if (apiResult.statusCode == "SUCESS") Employee = new Employee();
                            MessageBox.Show(string.Format("Delete user completed with code {0}, description {1} ", apiResult.statusCode, apiResult.statusDescription));
                        }
                        else
                        {
                            MessageBox.Show(string.Format("Error:{0}({1}).", resp.StatusDescription, resp.StatusCode), "Error");
                        }
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(string.Format(ex.StackTrace), "Error");
                    }

                }));
            }
        }

        private ICommand _syncEventsCommand;
        public ICommand SyncEventsCommand
        {
            get
            {
                return _syncEventsCommand ?? (_syncEventsCommand = new CommandHandler(() =>
                {
                    try
                    {
                        if (string.IsNullOrWhiteSpace(_token))
                        {
                            MessageBox.Show("Not connected");
                            return;
                        }
                        string param = string.IsNullOrWhiteSpace(_attendanceEvent.employeeCode) ? "" : "?code=" + _attendanceEvent.employeeCode;
                        string dateRagne = string.Format("from={0}&to={1}", Uri.EscapeDataString(SyncDate.Date.ToString("yyyy-MM-dd HH:mm:ss")), Uri.EscapeDataString(SyncDate.AddDays(1).Date.ToString("yyyy-MM-dd HH:mm:ss")));
                        param += (string.IsNullOrEmpty(param) ? "?" : "&") + dateRagne;
                        WebRequest req = WebRequest.Create(_serverUrl + "/api/ncheck/event"
                            + param);
                        SetSSLValidator(req);
                        req.Method = "GET";
                        req.Headers["Authorization"] = "Bearer " + _token;
                        req.ContentType = "application/json";
                        HttpWebResponse resp = req.GetResponse() as HttpWebResponse;
                        if (resp.StatusCode == HttpStatusCode.OK)
                        {
                            var result = new StreamReader(resp.GetResponseStream()).ReadToEnd();
                            var apiResult = JsonConvert.DeserializeObject<APIResult<List<AttendanceEvent>>>(result);
                            AttendanceEventList = apiResult.returnValue;
                            MessageBox.Show(string.Format("Sync events completed with code {0}, description {1} ", apiResult.statusCode, apiResult.statusDescription));
                        }
                        else
                        {
                            MessageBox.Show(string.Format("Error:{0}({1}).", resp.StatusDescription, resp.StatusCode), "Error");
                        }

                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(string.Format(ex.StackTrace), "Error");
                    }
                }));
            }
        }


        private ICommand _biometricAttendanceCommand;
        public ICommand BiometricAttendanceCommand
        {
            // RegisterEventData eventData
            //string url = API_URI_BASE + "api/device/v5/registerevent";
            //var result = this.RestTemplate.Exchange<NCheckResponse<EventReport>>(url, HttpMethod.POST, new HttpEntity(eventData));
            //return result.Body;
            get
            {
                return _markAttendanceCommand ?? (_markAttendanceCommand = new CommandHandler(() =>
                {
                    try
                    {
                        if (string.IsNullOrWhiteSpace(_token))
                        {
                            MessageBox.Show("Not connected");
                            return;
                        }
                        if (string.IsNullOrWhiteSpace(_attendanceEvent.employeeCode))
                        {
                            MessageBox.Show("Event employee code is required");
                            return;
                        }
                        WebRequest req = WebRequest.Create(_serverUrl + "api/device/v5/registerevent");
                        SetSSLValidator(req);
                        req.Method = "POST";
                        req.Headers["Authorization"] = "Bearer " + _token;
                        req.ContentType = "application/json; charset=utf-8";
                        var settings = new JsonSerializerSettings
                        {
                            DateFormatString = "yyyy-MM-dd HH:mm:ss",
                            DateTimeZoneHandling = DateTimeZoneHandling.Utc
                        };
                        var postData = JsonConvert.SerializeObject(new List<AttendanceEvent> { AttendanceEvent }, settings);
                        var data = Encoding.UTF8.GetBytes(postData);
                        req.ContentLength = data.Length;
                        using (var stream = req.GetRequestStream())
                        {
                            stream.Write(data, 0, data.Length);
                        }
                        HttpWebResponse resp = req.GetResponse() as HttpWebResponse;
                        if (resp.StatusCode == HttpStatusCode.OK)
                        {
                            var result = new StreamReader(resp.GetResponseStream()).ReadToEnd();
                            var apiResult = JsonConvert.DeserializeObject<APIResult<List<AttendanceEvent>>>(result);
                            MessageBox.Show(string.Format("Update event completed with code {0}, description {1} ", apiResult.statusCode, apiResult.statusDescription));
                        }
                        else
                        {
                            MessageBox.Show(string.Format("Error:{0}({1}).", resp.StatusDescription, resp.StatusCode), "Error");
                        }
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(string.Format(ex.StackTrace), "Error");
                    }

                }));
            }
        }

        private ICommand _markAttendanceCommand;
        public ICommand MarkAttendanceCommand
        {
            get
            {
                return _markAttendanceCommand ?? (_markAttendanceCommand = new CommandHandler(() =>
                {
                    try
                    {
                        if (string.IsNullOrWhiteSpace(_token))
                        {
                            MessageBox.Show("Not connected");
                            return;
                        }
                        if (string.IsNullOrWhiteSpace(_attendanceEvent.employeeCode))
                        {
                            MessageBox.Show("Event employee code is required");
                            return;
                        }
                        WebRequest req = WebRequest.Create(_serverUrl + "/api/ncheck/event");
                        SetSSLValidator(req);
                        req.Method = "POST";
                        req.Headers["Authorization"] = "Bearer " + _token;
                        req.ContentType = "application/json; charset=utf-8";
                        var settings = new JsonSerializerSettings
                        {
                            DateFormatString = "yyyy-MM-dd HH:mm:ss",
                            DateTimeZoneHandling = DateTimeZoneHandling.Utc
                        };
                        var postData = JsonConvert.SerializeObject(new List<AttendanceEvent> { AttendanceEvent }, settings);
                        var data = Encoding.UTF8.GetBytes(postData);
                        req.ContentLength = data.Length;
                        using (var stream = req.GetRequestStream())
                        {
                            stream.Write(data, 0, data.Length);
                        }
                        HttpWebResponse resp = req.GetResponse() as HttpWebResponse;
                        if (resp.StatusCode == HttpStatusCode.OK)
                        {
                            var result = new StreamReader(resp.GetResponseStream()).ReadToEnd();
                            var apiResult = JsonConvert.DeserializeObject<APIResult<List<AttendanceEvent>>>(result);
                            MessageBox.Show(string.Format("Update event completed with code {0}, description {1} ", apiResult.statusCode, apiResult.statusDescription));
                        }
                        else
                        {
                            MessageBox.Show(string.Format("Error:{0}({1}).", resp.StatusDescription, resp.StatusCode), "Error");
                        }
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(string.Format(ex.StackTrace), "Error");
                    }

                }));
            }
        }
        private ICommand _deleteAttendanceCommand;
        public ICommand DeleteAttendanceCommand
        {
            get
            {
                return _deleteAttendanceCommand ?? (_deleteAttendanceCommand = new CommandHandler(() =>
                {
                    try
                    {
                        if (string.IsNullOrWhiteSpace(_token))
                        {
                            MessageBox.Show("Not connected");
                            return;
                        }
                        if (string.IsNullOrWhiteSpace(_attendanceEvent.employeeCode))
                        {
                            MessageBox.Show("Event employee code is required");
                            return;
                        }
                        WebRequest req = WebRequest.Create(_serverUrl + "/api/ncheck/event");
                        SetSSLValidator(req);
                        req.Method = "DELETE";
                        req.Headers["Authorization"] = "Bearer " + _token;
                        req.ContentType = "application/json; charset=utf-8";
                        var settings = new JsonSerializerSettings
                        {
                            DateFormatString = "yyyy-MM-dd HH:mm:ss",
                            DateTimeZoneHandling = DateTimeZoneHandling.Utc
                        };
                        var postData = JsonConvert.SerializeObject(new List<AttendanceEvent> { AttendanceEvent }, settings);
                        var data = Encoding.UTF8.GetBytes(postData);
                        req.ContentLength = data.Length;
                        using (var stream = req.GetRequestStream())
                        {
                            stream.Write(data, 0, data.Length);
                        }
                        HttpWebResponse resp = req.GetResponse() as HttpWebResponse;
                        if (resp.StatusCode == HttpStatusCode.OK)
                        {
                            var result = new StreamReader(resp.GetResponseStream()).ReadToEnd();
                            var apiResult = JsonConvert.DeserializeObject<APIResult<List<AttendanceEvent>>>(result);
                            MessageBox.Show(string.Format("Delete event completed with code {0}, description {1} ", apiResult.statusCode, apiResult.statusDescription));
                        }
                        else
                        {
                            MessageBox.Show(string.Format("Error:{0}({1}).", resp.StatusDescription, resp.StatusCode), "Error");
                        }
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(string.Format(ex.StackTrace), "Error");
                    }
                }));
            }
        }

        private ICommand _copyAttendanceCommand;
        public ICommand CopyAttendanceCommand
        {
            get
            {
                return _copyAttendanceCommand ?? (_copyAttendanceCommand = new CommandHandler(() =>
                {
                    if (SelectedEvent != null) AttendanceEvent = SelectedEvent;
                    else MessageBox.Show("Please select an event first");
                }));
            }
        }
        private ICommand _resetAttendanceCommand;
        public ICommand ResetAttendanceCommand
        {
            get
            {
                return _resetAttendanceCommand ?? (_resetAttendanceCommand = new CommandHandler(() =>
                {
                    AttendanceEvent = new AttendanceEvent();
                }));
            }
        }
        public MainWindowViewModel()
        {

        }
        private class CommandHandler : ICommand
        {
            private Action _action;
            public CommandHandler(Action action)
            {
                _action = action;
            }

            public event EventHandler CanExecuteChanged
            {
                add { CommandManager.RequerySuggested += value; }
                remove { CommandManager.RequerySuggested -= value; }
            }
            public bool CanExecute(object parameter)
            {
                return true;
            }

            public void Execute(object parameter)
            {
                _action();
            }
        }
    }
}
