﻿using Neurotec.NCheck.PeripheralSDK;
using System;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using System.Threading;
using System.Windows;
using System.Windows.Media.Imaging;

namespace Neurotec.NCheck.SDK.Samples
{
    [Guid("0130A000-8261-4ECF-8C38-08D577CB2154")]
    public class DummyFingerprintService : CaptureDeviceService
    {
        private string DeviceRef=null;
        public DummyFingerprintService()
        {

        }

        public override string GetName()
        {
            return "Dummy Fingerprint Scanner";
        }

        public override CaptureDataType[] GetCaptureTypes()
        {
            CaptureDataType[] captureTypes = { CaptureDataType.FINGERPRINT };
            return captureTypes;
        }

        public override System.Windows.Point[] GetImageSizes()
        {
            System.Windows.Point[] resolutions = { mDefaultResolution, DeviceRef==null ? new System.Windows.Point(256, 256) : new System.Windows.Point(384, 384) };
            return resolutions;
        }

        public override void ShowMessage(DisplayMessageType type, string msg)
        {
            MessageBox.Show(msg);
        }

        public override void ShowResults(string name, int inout, string time1, string time2)
        {
            MessageBox.Show(name + "\r\nIN:" + time1 + "\r\nOUT:" + time2);
        }

        public override void Connect()
        {
            DeviceRef = GetDeviceRef();
            CancelCapture(CaptureDataType.FINGERPRINT);
            this.statusChanged(CaptureDeviceStatus.READY);
            GetInfo(); // Send device information on start
        }

        Timer mTimer = null;
        System.Windows.Point mDefaultResolution = new System.Windows.Point(512, 512);
        System.Windows.Point mCurrentResolution = new System.Windows.Point(512, 512);
		public override void Capture(CaptureDataType? dataType, System.Windows.Point resolution)
        {
            if (resolution != null && GetImageSizes().Contains(resolution)) mCurrentResolution = resolution;
            else mCurrentResolution = mDefaultResolution;
            if (mTimer == null)
            {
                this.statusChanged(CaptureDeviceStatus.CAPTURING);
                mTimer = new Timer(delegate (object sender) {
                    int width = (int)mCurrentResolution.X;
                    int height = (int)mCurrentResolution.Y;
                    // Define parameters used to create the BitmapSource.
                    int rawStride = (width * System.Windows.Media.PixelFormats.Gray16.BitsPerPixel + 7) / 8;
                    byte[] rawImage = new byte[rawStride * height];

                    // Initialize the image with data.
                    Random value = new Random();
                    value.NextBytes(rawImage);

                    // Create a BitmapSource.
                    BitmapSource bitmap = BitmapSource.Create(width, height,
                        96, 96, System.Windows.Media.PixelFormats.Gray16, null,
                        rawImage, rawStride);
				JpegBitmapEncoder encoder = new JpegBitmapEncoder();
				encoder.QualityLevel = 100;
					using (var ms = new MemoryStream())
					{
						encoder.Frames.Add(BitmapFrame.Create(bitmap));
						encoder.Save(ms);
						byte[] bit = ms.ToArray();
						sendCapturedData(CaptureDataType.FINGERPRINT, bit, null, null);
					}
                }, null, 1000, 1000);
            }

        }

        public override void CancelCapture(CaptureDataType? dataType)
        {
            if (mTimer != null)
            {
                this.statusChanged(CaptureDeviceStatus.READY);
                mTimer.Dispose();
                mTimer = null;
            }
        }
        public override void Disconnect()
        {
            CancelCapture(CaptureDataType.FINGERPRINT);
            this.statusChanged(CaptureDeviceStatus.NOT_READY);
        }

        public override void Configure(string devRef)
        {
            new DummyScanner.ConfigWindow().ShowDialog();
        }
    }
}
